
#include <cotopha.h>


//
// O[oIuWFNg
//
shared Window		screen ;
shared InputFilter	input ;

//
// O[o֐
//
WndSpriteCmd & GetCommand( int nTimeout = INFINITE ) ;
bool DispatchCommand( WndSpriteCmd & wscmd ) ;

enum	ExceptionCode
{
	Exit
} ;

void Run( void ) ;


//////////////////////////////////////////////////////////////////////////////
// L[uf
//////////////////////////////////////////////////////////////////////////////

class	CubeModel	: public PolygonModel
{
protected:
	Resource	m_rsTexture ;

public:
	void CreateCube( void ) ;

} ;


//////////////////////////////////////////////////////////////////////////////
// Gg|Cg
//////////////////////////////////////////////////////////////////////////////

int main()
{
	screen.CreateDisplay
		( "Cube View Sample",
			Window::Copperation::Window, 640, 480 ) ;
	screen.SetOptionalFuncFlag
		( screen.GetOptionalFuncFlag()
			| Window::Flag::AllowClose
			| Window::Flag::AllowMinimize
			| Window::Flag::VariableWindowSize
			| Window::Flag::AllowMaximize ) ;
	screen.ShowCursor( true ) ;
	screen.EnableCommandQueue( true ) ;

	input.LoadInputFilter( "input.xml" ) ;
	input.OpenFilter( InputFilter::Type::Below, screen ) ;

	Run() ;

	screen.CloseDisplay() ;

	return	0 ;
}


//////////////////////////////////////////////////////////////////////////////
// R}h擾
//////////////////////////////////////////////////////////////////////////////

WndSpriteCmd & GetCommand( int nTimeout )
{
	WndSpriteCmd &	wscmd = WndSpriteCmd ;
	if ( screen.GetCommand( wscmd, nTimeout ) == eslErrSuccess )
	{
		if ( !DispatchCommand( wscmd ) )
		{
			return	wscmd ;
		}
	}
	return	null ;
}


//////////////////////////////////////////////////////////////////////////////
// VXeR}h
//////////////////////////////////////////////////////////////////////////////

bool DispatchCommand( WndSpriteCmd & wscmd )
{
	switch ( wscmd.strID )
	{
	case	SysCommandId::AppExit:
		throw	ExceptionCode::Exit ;
	}
	return	false ;
}



//////////////////////////////////////////////////////////////////////////////
// C[v
//////////////////////////////////////////////////////////////////////////////

void Run( void )
{
	//
	// IuWFNg
	//
	RenderSprite	render ;
	ModelJoint		jointRoot ;
	CubeModel		model ;
	//
	model.CreateCube() ;
	//
	// f\ݒ
	//
	int			iSubJoint = jointRoot.AddSubJoint() ;
	ModelJoint&	jointModel = jointRoot[iSubJoint] ;
	//
	jointModel.AddModelRef( model ) ;
	jointModel.InitializeMatrix() ;
	jointModel.RotateOnX( -45 ) ;
	jointModel.MagnifyByVector( 100, 100, 100 ) ;
	//
	// ݒ
	//
	LightEntry[]	lights ;
	//
	// 
	lights[0].nLightType = E3D_AMBIENT_LIGHT ;
	lights[0].rgbColor = 0x202020 ;
	//
	// 
	lights[1].nLightType = E3D_VECTOR_LIGHT ;
	lights[1].rgbColor = 0xFFFFFF ;
	lights[1].rBrightness = 1.0 ;
	lights[1].vecLight.x = 1 ;
	lights[1].vecLight.y = 1 ;
	lights[1].vecLight.z = 1 ;
	//
	// 3D \
	//
	screen.CreateZBuffer() ;
	screen.SetScreenPosition( 320, 240, 1024 ) ;
	//
	screen.Lock() ;
	//
	render.Initialize( screen, 0 ) ;
	//
	// J
	render.SetViewPoint( Vector( 0, 0, -1024 ), Vector( 0, 0, 0 ), 0 ) ;
	//
	// [gWCg
	render.AttachRootJoint( jointRoot ) ;
	render.UpdateRendering() ;
	//
	// 
	render.SetLightEntries( lights ) ;
	//
	// Aj[Vݒ
	Bezier3D	bzRotate ;
	bzRotate.SetLine( 0, 0, 0, 0, 360, 0, 1, 1 ) ;
	jointRoot.SetBezierCurve( , bzRotate ) ;
	jointRoot.BeginActivation( 10000, AnimationLoopType::Loop ) ;
	//
	screen.Unlock() ;
	//
	// ̖[v
	//
	try
	{
		for ( ; ; )
		{
			GetCommand() ;
		}
	}
	catch ( int code )
	{
	}
	//
	// I
	//
	screen.Lock() ;
	render.AttachRootJoint( null ) ;
	render.Release() ;
	screen.Unlock() ;
}



//////////////////////////////////////////////////////////////////////////////
// L[uf
//////////////////////////////////////////////////////////////////////////////

data	CubeVertices<double[][]>
{
	// W_
	upper_square =
	{
		{ -1.0, -1.0, -1.0 },
		{ -1.0, -1.0,  1.0 },
		{  1.0, -1.0, -1.0 },
		{  1.0, -1.0,  1.0 },
	},
	// W_
	under_square =
	{
		{  1.0,  1.0, -1.0 },
		{  1.0,  1.0,  1.0 },
		{ -1.0,  1.0, -1.0 },
		{ -1.0,  1.0,  1.0 },
	},
	// ʒ_
	side_square =
	{
		{ -1.0, -1.0,  1.0 },
		{ -1.0, -1.0, -1.0 },
		{  1.0, -1.0, -1.0 },
		{  1.0, -1.0,  1.0 },
		{ -1.0, -1.0,  1.0 },
		{ -1.0,  1.0,  1.0 },
		{ -1.0,  1.0, -1.0 },
		{  1.0,  1.0, -1.0 },
		{  1.0,  1.0,  1.0 },
		{ -1.0,  1.0,  1.0 },
	},
} ;

data	CubeTextureUVs<double[][]>
{
	// W
	upper_square =
	{
		{ 320.0        , 180.0 + 148.0 },
		{ 320.0        , 180.0 },
		{ 320.0 + 148.0, 180.0 + 148.0 },
		{ 320.0 + 148.0, 180.0 },
	},
	// W
	under_square =
	{
		{ 320.0 + 148.0, 32.0 + 148.0 },
		{ 320.0 + 148.0, 32.0 },
		{ 320.0        , 32.0 + 148.0 },
		{ 320.0        , 32.0 },
	},
	// 
	side_square =
	{
		{ 320.0 - 148.0, 180.0 + 148.0 },
		{ 320.0        , 180.0 + 148.0 },
		{ 320.0 + 148.0, 180.0 + 148.0 },
		{ 320.0 + 296.0, 180.0 + 148.0 },
		{ 320.0 + 444.0, 180.0 + 148.0 },
		{ 320.0 - 148.0, 180.0 + 296.0 },
		{ 320.0        , 180.0 + 296.0 },
		{ 320.0 + 148.0, 180.0 + 296.0 },
		{ 320.0 + 296.0, 180.0 + 296.0 },
		{ 320.0 + 444.0, 180.0 + 296.0 },
	},
} ;

void CubeModel::CreateCube( void )
{
	//
	// eNX`摜ǂݍ
	//
	m_rsTexture.LoadImage( "hatomimi.bmp" ) ;	// must be 32 bpp!
	//
	ImageInfo&	infTexture = m_rsTexture.GetImageInfo() ;
	if ( infTexture === null )
	{
		return ;
	}
	//
	// \ʑo^
	//
	SurfaceAttribute	sufattr ;
	sufattr.nShadingFlags = E3DSAF_GOURAUD_SHADE ;
	sufattr.rgbColorMul = 0xFFFFFF ;
	sufattr.rgbColorAdd = 0 ;
	sufattr.nAmbient = 0x60 ;
	sufattr.nDiffusion = 0xA0 ;
	sufattr.nSpecular = 0x40 ;
	//
	String	idAttr =
		RegisterTextureAttribute( m_rsTexture, sufattr ) ;
	//
	// W𐶐
	//
	const double[][]&	CubeUpperVertices = CubeVertices["upper_square"] ;
	const double[][]&	CubeUpperUVs = CubeTextureUVs["upper_square"] ;
	float[3]* pvVertex = new float[sizeof(CubeUpperVertices)][3] ;
	float[2]* pvUVMap = new float[sizeof(CubeUpperUVs)][2] ;
	//
	for ( e : CubeUpperVertices : i )
	{
		pvVertex[i][0] = e[0] ;
		pvVertex[i][1] = e[1] ;
		pvVertex[i][2] = e[2] ;
		pvUVMap[i][0] = CubeUpperUVs[i][0] ;
		pvUVMap[i][1] = CubeUpperUVs[i][1] ;
	}
	//
	AddTriangleStripPrimitive
		( idAttr, 2,
			PolygonModel::GridType::AutoSmooth,
			pvVertex, null, pvUVMap ) ;
	//
	delete []	pvVertex ;
	delete []	pvUVMap ;
	//
	// ʂ𐶐
	//
	const double[][]&	CubeSideVertices = CubeVertices["side_square"] ;
	const double[][]&	CubeSideUVs = CubeTextureUVs["side_square"] ;
	pvVertex = new float[4][3] ;
	pvUVMap = new float[4][2] ;
	//
	for ( int i = 0; i < 4; i ++ )
	{
		// AddGridMeshPrimitive 1ŊȒPɐo邪
		// @eʂɓƗ@𗧂Ă邽߁A4ɕĎs
		int[]	nIndexes =
		{
			0, 1, 5, 6
		} ;
		for ( e : nIndexes : j )
		{
			pvVertex[j][0] = CubeSideVertices[i + e][0] ;
			pvVertex[j][1] = CubeSideVertices[i + e][1] ;
			pvVertex[j][2] = CubeSideVertices[i + e][2] ;
			pvUVMap[j][0] = CubeSideUVs[i + e][0] ;
			pvUVMap[j][1] = CubeSideUVs[i + e][1] ;
		}
		AddGridMeshPrimitive
			( idAttr, 1, 1,
				PolygonModel::GridType::AutoSmooth,
				pvVertex, null, pvUVMap ) ;
	}
	delete []	pvVertex ;
	delete []	pvUVMap ;
	//
	// W𐶐
	//
	const double[][]&	CubeUnderVertices = CubeVertices["under_square"] ;
	const double[][]&	CubeUnderUVs = CubeTextureUVs["under_square"] ;
	pvVertex = new float[sizeof(CubeUnderVertices)][3] ;
	pvUVMap = new float[sizeof(CubeUnderUVs)][2] ;
	//
	for ( e : CubeUnderVertices : i )
	{
		pvVertex[i][0] = e[0] ;
		pvVertex[i][1] = e[1] ;
		pvVertex[i][2] = e[2] ;
		pvUVMap[i][0] = CubeUnderUVs[i][0] ;
		pvUVMap[i][1] = CubeUnderUVs[i][1] ;
	}
	//
	AddTriangleStripPrimitive
		( idAttr, 2,
			PolygonModel::GridType::AutoSmooth,
			pvVertex, null, pvUVMap ) ;
	//
	delete []	pvVertex ;
	delete []	pvUVMap ;
}


